#include "components.hpp"

/**************************** ImpedanceDevice Class ***************************/
ImpedanceDevice::ImpedanceDevice(Designator dsgntr, Node *nd1, Node *nd2, double vl)
    : Component(dsgntr, nd1, nd2), value(vl) {}

void ImpedanceDevice::update_value(double value)
{
  this->value = value;
}

double ImpedanceDevice::get_value()
{
  return value;
}

double ImpedanceDevice::get_voltage()
{
  return voltage;
}

/******************************* Resistor Class *******************************/
Resistor::Resistor(Node *nd1, Node *nd2, double vl)
    : ImpedanceDevice(Designator::Resistor, nd1, nd2, vl) {}

pair<double, double> Resistor::update_V_I(vector<double> V)
{
  voltage = calc_voltage(V);
  current = voltage / value;

  return make_pair(voltage, current);
}

double Resistor::get_conductance()
{
  return 1/value;
}

/************************ DynamicImpedanceDevice Class ************************/
DynamicImpedanceDevice::DynamicImpedanceDevice(Designator dsgntr, Node *nd1,
    Node *nd2, double vl) : ImpedanceDevice(dsgntr, nd1, nd2, vl) {}

void DynamicImpedanceDevice::set_eq_source(InputSource *eq_source) {}

void DynamicImpedanceDevice::set_parallel2Vsrc(bool parallel2Vsrc)
{
  this->parallel2Vsrc = parallel2Vsrc;
}

bool DynamicImpedanceDevice::get_parallel2Vsrc()
{
  return parallel2Vsrc;
}

void DynamicImpedanceDevice::update_current(double current, double prev_current)
{
  this->prev_current = prev_current;
  this->current = current;
}

InputSource *DynamicImpedanceDevice::get_eq_source()
{
  throw logic_error("SHOULD NOT EVER RUN");
  return NULL;
}

/******************************* Capacitor Class ******************************/
Capacitor::Capacitor(Node *nd1, Node *nd2, double vl)
    : DynamicImpedanceDevice(Designator::Capacitor, nd1, nd2, vl) {}

pair<double, double> Capacitor::update_V_I(vector<double> V, double t_step)
{
  prev_voltage = voltage;
  prev_current = current;
  voltage = calc_voltage(V);

  current = (voltage - prev_voltage) * value / t_step;

  return make_pair(voltage, current);
}

void Capacitor::set_eq_source(InputSource *eq_source)
{
  this->eq_source = eq_source;
}

InputSource * Capacitor::get_eq_source()
{
  return eq_source;
}

/******************************* Inductor Class *******************************/
Inductor::Inductor(Node *nd1, Node *nd2, double vl)
    : DynamicImpedanceDevice(Designator::Inductor, nd1, nd2, vl) {}

pair<double, double> Inductor::update_V_I(vector<double> V, double t_step)
{
  prev_voltage = voltage;
  prev_current = current;

  voltage = calc_voltage(V);
  current += voltage * t_step / value;

  return make_pair(voltage, current);
}

void Inductor::set_eq_source(InputSource *eq_source)
{
  this->eq_source = eq_source;
}

InputSource * Inductor::get_eq_source()
{
  return eq_source;
}