#include "components.hpp"

using namespace std;

NonLinearDevice::NonLinearDevice(
    Designator dsgntr, Node *nd1, Node *nd2, Node *nd3, string vl)
    : Component(dsgntr, nd1, nd2, nd3), value(vl) {}

NonLinearDevice::NonLinearDevice(
    Designator dsgntr, Node *nd1, Node *nd2, string vl)
    : Component(dsgntr, nd1, nd2), value(vl) {}

void NonLinearDevice::set_eq_components(Resistor *eq_resistor,
                                        CurrentSource *eq_i_source)
{
  this->eq_i_source = eq_i_source;
  this->eq_resistor = eq_resistor;
}

void NonLinearDevice::set_eq_components(NonLinearDevice *eq_diode,
                                        CurrentSource *eq_i_source)
{
  this->eq_i_source = eq_i_source;
  this->eq_diode = eq_diode;
}

void NonLinearDevice::update_eq_components(vector<double> V)
{
  throw logic_error("SHOULD NOT EVER RUN");
}

Resistor *NonLinearDevice::get_eq_resistor()
{
  return eq_resistor;
}

CurrentSource *NonLinearDevice::get_eq_i_source()
{
  return eq_i_source;
}

NonLinearDevice *NonLinearDevice::get_eq_diode()
{
  return eq_diode;
}

string NonLinearDevice::get_type() const
{
  if (value == "D")
    return "Silicon diode";
  else if (value == "NPN")
    return "NPN BJT";
  else if (value == "PNP")
    return "PNP BJT";
  else
    return "N/A";
}

Transistor::Transistor(Node *nd1, Node *nd2, Node *nd3, string vl)
: NonLinearDevice(Designator::Transistor, nd1, nd2, nd3, vl) {}

double Transistor::calc_voltage(vector<double> V)
{
  vector<double> V_aug = V;
  reverse(V_aug.begin(), V_aug.end());
  V_aug.push_back(0);
  reverse(V_aug.begin(), V_aug.end());

  return V_aug[node2->id] - V_aug[node3->id];
}

void Transistor::update_eq_components(vector<double> V)
{
  double V_be = max(calc_voltage(V), 0.001);

  if (eq_diode == NULL)
    if (value == "NPN")
      eq_diode = new Diode(node2, node3);
    else
      eq_diode = new Diode(node3, node2);

  eq_diode->update_eq_components(V);

  double src_I = TRANSISTOR_B * eq_diode->get_current();
  // double src_I = 0.003;

  if (eq_i_source == NULL)
    if (value == "NPN")
      eq_i_source = new CurrentSource(node1, node3, src_I);
    else
      eq_i_source = new CurrentSource(node3, node1, src_I);
  else
    eq_i_source->update_offset(src_I);
}

Diode::Diode(Node *nd1, Node *nd2)
: NonLinearDevice(Designator::Diode, nd1, nd2, "D") {}

void Diode::update_eq_components(vector<double> V)
{
  voltage = max(calc_voltage(V), 0.01);
  current = DIODE_Is * (exp(voltage / DIODE_VT) - 1);

  double resistance = DIODE_VT / (DIODE_Is * exp(voltage / DIODE_VT));
  double src_current = current - voltage / resistance;

  if (eq_i_source == NULL)
    eq_i_source = new CurrentSource(node1, node2, src_current);
  else
    eq_i_source->update_offset(src_current);

  if (eq_resistor == NULL)
    eq_resistor = new Resistor(node1, node2, resistance);
  else
    eq_resistor->update_value(resistance);
}
