#ifndef CIRCUIT_H
#define CIRCUIT_H

#include "universal.hpp"
#include "types_etc.hpp"
#include "parser.hpp"

#define CIRCUIT_MAX_CURRENT 10e3

class Circuit
{
private:
  double t_step;

  ConstantSourceVector cnst_src_vec;
  ConductanceMatrix cndct_mat;
  SuperNodes s_nodes;
  Node * reference = NULL;
  Nodes nodes;

  bool check_if_supernodes_connected(SuperNode *s_node_1, SuperNode *s_node_2);
  double calc_node_pair_conductance(Node* node1, Node* node2);

public:
  DynamicImpedanceDevices d_impdnc_devs;
  InputSources in_srcs;
  Resistors resistors;
  NonLinearDevices nn_devs;

  const int nodes_num;

  // Circuit(ImpedanceDevices impdnc_devs, InputSources in_srcs, Nodes nodes);
  Circuit(Parser *parser);

  ConstantSourceVector get_const_source_vec();
  ConductanceMatrix get_conductance_mat();
  InputSources get_input_sources();
  SuperNodes get_supernodes();
  Nodes get_nodes();

  double calc_diode_eq_resistor(double V);
  double calc_diode_eq_i_source(double V);
  // // double calc_inductor_eq_resistor(DynamicImpedanceDevice *indctr);
  // double calc_inductor_eq_i_source(DynamicImpedanceDevice *indctr, double V);
  // // double calc_capacitor_eq_resistor(DynamicImpedanceDevice *cap);
  // double calc_capacitor_eq_v_source(DynamicImpedanceDevice *cap, double I);
  void calc_dynamic_eq_sources(vector<double> V);
  void calc_non_linear_eq_components(vector<double> V);
  void set_unset_inductor_eq_shortcircuit();
  // void calc_eq_components(DynamicImpedanceDevice *d_impdnc_dev, double V);
  // void calc_eq_components(NonLinearDevice *nn_dev, double V);

  void calc_dynamic_parallel2Vsrc();

  void calc_bias_point_v_sources_currents(vector<double> V);
  void calc_bias_point_inductors_currents(vector<double> V);

  void update_with_simple_nodes(Node* node);
  void update_const_with_simple_nodes(Node* node, double t);
  void update_with_volt_source(InputSource* Vsrc, int row);
  void update_const_with_volt_source(InputSource* Vsrc, int row, double t);
  bool update_with_supernode(SuperNode *s_node, int row);
  bool update_const_with_supernode(SuperNode *s_node, int row, double t);
  void create_supernodes(InputSources aug_in_srcs);
  ConstantSourceVector update_constant_source_vec(double t);
  ConductanceMatrix update_conductance_mat();
};

#endif