#include <iterator>

#include "universal.hpp"

#include "node.hpp"
#include "node.cpp"

#include "components.hpp"
#include "Component.cpp"
#include "ImpedanceDevice.cpp"
#include "InputSource.cpp"
#include "NonLinearDevice.cpp"

#include "utilities.cpp"

#include "circuit.hpp"
#include "circuit.cpp"

#include "parser.hpp"
#include "parser.cpp"

#include "solver.cpp"

int main()
{
  Parser parser("parser.txt");

  cout << parser.nodes << endl;
  cout << endl;
  cout << parser.resistors << endl;
  cout << endl;
  cout << parser.d_impdnc_devs << endl;
  cout << endl;
  cout << parser.in_srcs << endl;
  cout << endl;
  cout << parser.nn_devs << endl;

  Circuit circuit(&parser);

  cout << "cond mat" << endl;
  cout << circuit.get_conductance_mat() << endl;
  cout << "const mat" << endl;
  cout << circuit.get_const_source_vec() << endl;

  Solver solver(&circuit, parser.t_stop, parser.t_step);
  cout << solver.get_conductance_matrix() << endl;
  solver.simulate();
  auto solution = solver.get_solution();

  ofstream output_file("./solution.txt");
  if (output_file.is_open()) {
    output_file << solution << '\n';
  }

  return 0;
}