#include "node.hpp"

Node::Node(int node_id, bool connected_to_Vsrc)
{
  id = node_id;
  update_connected_to_Vsrc(connected_to_Vsrc);
}

Node::Node(const Node &node)
{
  id = node.id;
  update_connected_to_Vsrc(node.connected_to_Vsrc);
  neighbors = node.neighbors;
}

Node::Node() = default;

bool Node::operator==(const Node &node) const
{
  return (id == node.id);
}

bool Node::operator!=(const Node &node) const
{
  return (id != node.id);
}

bool Node::is_reference()
{
  return id == 0;
}

bool Node::is_connected_to_Vsrc()
{
  return connected_to_Vsrc;
}

void Node::update_connected_to_Vsrc(bool connected_to_Vsrc)
{
  this->prev_connected_to_Vsrc = this->connected_to_Vsrc;
  this->connected_to_Vsrc = connected_to_Vsrc;
}

void Node::restore_is_connected_to_Vsrc()
{
  this->connected_to_Vsrc = this->prev_connected_to_Vsrc;
}

bool Node::has_neighbor(Node *node)
{
  return neighbors[node->id];
}

vector<bool> Node::get_neighbors()
{
  return neighbors;
}

void Node::update_neighbors(Node *neighbor)
{
  extend_neighbors(neighbor->id + 1);
  neighbors[neighbor->id] = true;
}

void Node::extend_neighbors(int nodes_num)
{
  int ns = neighbors.size();
  if (nodes_num > ns)
    for (int i = 0; i < (nodes_num - ns); i++)
      neighbors.push_back(false);
}