#ifndef UTILITIES_H
#define UTILITIES_H

#include "universal.hpp"
#include "types_etc.hpp"

std::ostream &operator<<(std::ostream &os, std::vector<Eigen::VectorXd> vec)
{
  for (auto v : vec)
    os << v << std::endl << "---" << std::endl;

  return os;
}

template <typename T>
std::ostream &operator<<(std::ostream &os, std::vector<T> vec)
{
  for (auto v : vec)
    os << v << " ";

  return os;
}

std::ostream &operator<<(std::ostream &os, Node &node)
{
  os << "Node ID: " << node.id << " is";
  if (! node.is_connected_to_Vsrc())
    os << " not";
  os << " connected to Voltage Source and has neighbors: ";
  os << node.get_neighbors();

  return os;
}

std::ostream &operator<<(std::ostream &os, ConductanceMatrix cndct_mat)
{
  for (auto &cndct_row : cndct_mat) {
    for (auto &cndct : cndct_row) {
      os << cndct << " ";
    }
    os << std::endl;
  }

  return os;
}

std::ostream &operator<<(std::ostream &os, ConstantSourceVector cnst_src_vec)
{
  for (auto &cnst_src_item : cnst_src_vec)
    os << cnst_src_item << " ";

  return os;
}

template <typename T>
std::ostream &operator<<(std::ostream &os, std::vector<T *> &vec)
{
  for (int i = 0; i < vec.size(); i++) {
    os << *(vec[i]);
    if (i < vec.size()-1)
      os << endl;
  }

  return os;
}

std::ostream &operator<<(std::ostream &os, const Designator dsgntr)
{
  switch (dsgntr)
  {
  case Designator::Voltage:
    os << "Voltage";
    break;
  case Designator::Current:
    os << "Current";
    break;
  case Designator::Capacitor:
    os << "Capacitor";
    break;
  case Designator::Inductor:
    os << "Inductor";
    break;
  case Designator::Resistor:
    os << "Resistor";
    break;
  case Designator::Transistor:
    os << "Transistor";
    break;
  case Designator::Diode:
    os << "Diode";
    break;
  }

  return os;
}

std::ostream &operator<<(std::ostream &os, InputSource &in_src)
{
  os << in_src.get_designator();
  os << " source with offset " << in_src.get_offset();
  os << " amplitude " << in_src.get_amplitude();
  os << " and frequency " << in_src.get_frequency();
  os << ", connected to nodes " << (in_src.get_node1())->id;
  os << " and " << (in_src.get_node2())->id;

  return os;
}

std::ostream &operator<<(std::ostream &os, Resistor &resistor)
{
  os << resistor.get_designator();
  os << " with conductance of " << resistor.get_conductance();
  os << ", connected to nodes " << (resistor.get_node1())->id;
  os << " and " << (resistor.get_node2())->id;

  return os;
}

std::ostream &operator<<(std::ostream &os, DynamicImpedanceDevice &d_impdnc_dev)
{
  os << d_impdnc_dev.get_designator();
  os << " with value (C or L) of " << d_impdnc_dev.get_value();
  os << ", connected to nodes " << (d_impdnc_dev.get_node1())->id;
  os << " and " << (d_impdnc_dev.get_node2())->id;

  return os;
}

std::ostream &operator<<(std::ostream &os, NonLinearDevice &nn_dev)
{
  os << nn_dev.get_designator();
  os << " of type: " << nn_dev.get_type();
  os << ", connected to nodes " << (nn_dev.get_node1())->id;
  if (nn_dev.get_designator() == Designator::Transistor) {
    os << ", " << (nn_dev.get_node2())->id;
    // os << " and " << (nn_dev.get_node3())->id;
  } else {
    os << " and " << (nn_dev.get_node2())->id;
  }

  return os;
}

#endif